@@ -1,5 +1,5 @@
 /*
- * * Copyright (C) 2009 Ali <aliov@xfce.org>
+ * * Copyright (C) 2009-2011 Ali <aliov@xfce.org>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -36,6 +36,7 @@
 #include "xfpm-xfconf.h"
 #include "xfpm-notify.h"
 #include "xfpm-config.h"
+#include "xfpm-button.h"
 #include "xfpm-enum-glib.h"
 #include "xfpm-enum-types.h"
 #include "xfpm-debug.h"
@@ -51,6 +52,7 @@ struct XfpmBatteryPrivate
 {
     XfpmXfconf             *conf;
     XfpmNotify		   *notify;
+    XfpmButton             *button;
     DBusGProxy             *proxy;
     DBusGProxy 		   *proxy_prop;
     
@@ -65,9 +67,10 @@ struct XfpmBatteryPrivate
     gint64		    time_to_full;
     gint64		    time_to_empty;
 
-    const gchar            *backend_iface_device; /*upower or devkit*/
+    const gchar            *battery_name;
     
     gulong		    sig;
+    gulong		    sig_bt;
 };
 
 enum
@@ -91,17 +94,27 @@ G_DEFINE_TYPE (XfpmBattery, xfpm_battery, GTK_TYPE_STATUS_ICON)
 static const gchar * G_GNUC_CONST
 xfpm_battery_get_icon_index (XfpmDeviceType type, guint percent)
 {
-    if (percent < 10) {
+    if (percent < 10) 
+    {
         return "000";
-    } else if (percent < 30) {
+    } 
+    else if (percent < 30) 
+    {
         return ( (type == XFPM_DEVICE_TYPE_BATTERY || type == XFPM_DEVICE_TYPE_UPS) ? "020" : "030");
-    } else if (percent < 50) {
+    } 
+    else if (percent < 50) 
+    {
         return ( (type == XFPM_DEVICE_TYPE_BATTERY || type == XFPM_DEVICE_TYPE_UPS ) ? "040" : "030");
-    } else if (percent < 70) {
+    } 
+    else if (percent < 70) 
+    {
         return "060";
-    } else if (percent < 90) {
+    } 
+    else if (percent < 90) 
+    {
         return ((type == XFPM_DEVICE_TYPE_BATTERY || type == XFPM_DEVICE_TYPE_UPS) ? "080" : "060");
     }
+    
     return "100";
 }
 
@@ -138,23 +151,129 @@ xfpm_battery_refresh_visible (XfpmBattery *battery)
 }
 
 
-static const gchar * G_GNUC_CONST
-xfpm_battery_get_message_from_battery_state (XfpmDeviceState state, gboolean ac_online)
+/*
+ * Taken from gpm
+ */
+static gchar *
+xfpm_battery_get_time_string (guint seconds)
 {
-    switch (state)
+    char* timestring = NULL;
+    gint  hours;
+    gint  minutes;
+
+    /* Add 0.5 to do rounding */
+    minutes = (int) ( ( seconds / 60.0 ) + 0.5 );
+
+    if (minutes == 0) 
     {
-	case XFPM_DEVICE_STATE_FULLY_CHARGED:
-	    return _("Your battery is fully charged");
-	    break;
-	case XFPM_DEVICE_STATE_CHARGING:
-	    return  _("Your battery is charging");
-	    break;
-	case XFPM_DEVICE_STATE_DISCHARGING:
-	    return  ac_online ? _("Your battery is discharging"): _("System is running on battery power");
-	    break;
-	default:
-	    return NULL;
+	timestring = g_strdup (_("Unknown time"));
+	return timestring;
     }
+
+    if (minutes < 60) 
+    {
+	timestring = g_strdup_printf (ngettext ("%i minute",
+			              "%i minutes",
+				      minutes), minutes);
+	return timestring;
+    }
+
+    hours = minutes / 60;
+    minutes = minutes % 60;
+
+    if (minutes == 0)
+	timestring = g_strdup_printf (ngettext (
+			    "%i hour",
+			    "%i hours",
+			    hours), hours);
+    else
+	/* TRANSLATOR: "%i %s %i %s" are "%i hours %i minutes"
+	 * Swap order with "%2$s %2$i %1$s %1$i if needed */
+	timestring = g_strdup_printf (_("%i %s %i %s"),
+			    hours, ngettext ("hour", "hours", hours),
+			    minutes, ngettext ("minute", "minutes", minutes));
+    return timestring;
+}
+
+static gchar *
+xfpm_battery_get_message_from_battery_state (XfpmBattery *battery)
+{
+    gchar *msg  = NULL;
+    
+    
+    if (battery->priv->type == XFPM_DEVICE_TYPE_BATTERY || battery->priv->type == XFPM_DEVICE_TYPE_UPS)
+    {
+	switch (battery->priv->state)
+	{
+	    case XFPM_DEVICE_STATE_FULLY_CHARGED:
+		msg = g_strdup_printf (_("Your %s is fully charged"), battery->priv->battery_name);
+		break;
+	    case XFPM_DEVICE_STATE_CHARGING:
+		msg = g_strdup_printf (_("Your %s is charging"), battery->priv->battery_name);
+		
+		if ( battery->priv->time_to_full != 0 )
+		{
+		    gchar *tmp, *est_time_str;
+		    tmp = g_strdup (msg);
+		    g_free (msg);
+		    
+		    est_time_str = xfpm_battery_get_time_string (battery->priv->time_to_full);
+		    
+		    msg = g_strdup_printf ("%s (%i%%)\n%s until is fully charged.", tmp, battery->priv->percentage, est_time_str);
+		    g_free (est_time_str);
+		    g_free (tmp);
+		}
+
+		break;
+	    case XFPM_DEVICE_STATE_DISCHARGING:
+		if (battery->priv->ac_online)
+		    msg =  g_strdup_printf (_("Your %s is discharging"), battery->priv->battery_name);
+		else
+		    msg =  g_strdup_printf (_("System is running on %s power"), battery->priv->battery_name);
+		    
+		    if ( battery->priv->time_to_empty != 0 )
+		    {
+			gchar *tmp, *est_time_str;
+			tmp = g_strdup (msg);
+			g_free (msg);
+			
+			est_time_str = xfpm_battery_get_time_string (battery->priv->time_to_empty);
+			
+			msg = g_strdup_printf ("%s (%i%%)\nEstimated time left is %s.", tmp, battery->priv->percentage, est_time_str);
+			g_free (tmp);
+			g_free (est_time_str);
+		    }
+		break;
+	    case XFPM_DEVICE_STATE_EMPTY:
+		msg = g_strdup_printf (_("Your %s is empty"), battery->priv->battery_name);
+		break;
+	    default:
+		break;
+	}
+	
+    }
+    else if (battery->priv->type >= XFPM_DEVICE_TYPE_MONITOR)
+    {
+	switch (battery->priv->state)
+	{
+	    case XFPM_DEVICE_STATE_FULLY_CHARGED:
+		msg = g_strdup_printf (_("Your %s is fully charged"), battery->priv->battery_name);
+		break;
+	    case XFPM_DEVICE_STATE_CHARGING:
+		msg = g_strdup_printf (_("Your %s is charging"), battery->priv->battery_name);
+		break;
+	    case XFPM_DEVICE_STATE_DISCHARGING:
+		msg =  g_strdup_printf (_("Your %s is discharging"), battery->priv->battery_name);
+		break;
+	    case XFPM_DEVICE_STATE_EMPTY:
+		msg = g_strdup_printf (_("Your %s is empty"), battery->priv->battery_name);
+		break;
+	    default:
+		break;
+	}
+    }
+    
+    return msg;
 }
 
 static void
@@ -212,27 +331,34 @@ xfpm_battery_refresh_icon (XfpmBattery *battery)
     gtk_status_icon_set_from_icon_name (GTK_STATUS_ICON (battery), icon_name);
 }
 
-static gboolean
-xfpm_battery_notify_idle (gpointer data)
+static void
+xfpm_battery_notify (XfpmBattery *battery)
 {
-    XfpmBattery *battery;
-    const gchar *message;
+    gchar *message = NULL;
     
-    battery = XFPM_BATTERY (data);
-    
-    message = xfpm_battery_get_message_from_battery_state (battery->priv->state, battery->priv->ac_online);
+    message = xfpm_battery_get_message_from_battery_state (battery);
     
     if ( !message )
-	return FALSE;
+	return;
 	
     xfpm_notify_show_notification (battery->priv->notify, 
-				   _("Xfce power manager"), 
+				   _("Power Manager"), 
 				   message, 
 				   gtk_status_icon_get_icon_name (GTK_STATUS_ICON (battery)),
 				   8000,
 				   FALSE,
 				   XFPM_NOTIFY_NORMAL,
 				   GTK_STATUS_ICON (battery));
+    
+    g_free (message);
+}
+
+static gboolean
+xfpm_battery_notify_idle (gpointer data)
+{
+    XfpmBattery *battery;
+    battery = XFPM_BATTERY (data);
+    xfpm_battery_notify (battery);
     return FALSE;
 }
 
@@ -265,65 +391,18 @@ xfpm_battery_notify_state (XfpmBattery *battery)
     }
 }
 
-/*
- * Taken from gpm
- */
-static gchar *
-xfpm_battery_get_time_string (guint seconds)
-{
-    char* timestring = NULL;
-    gint  hours;
-    gint  minutes;
-
-    /* Add 0.5 to do rounding */
-    minutes = (int) ( ( seconds / 60.0 ) + 0.5 );
-
-    if (minutes == 0) 
-    {
-	timestring = g_strdup (_("Unknown time"));
-	return timestring;
-    }
-
-    if (minutes < 60) 
-    {
-	timestring = g_strdup_printf (ngettext ("%i minute",
-			              "%i minutes",
-				      minutes), minutes);
-	return timestring;
-    }
-
-    hours = minutes / 60;
-    minutes = minutes % 60;
-
-    if (minutes == 0)
-	timestring = g_strdup_printf (ngettext (
-			    "%i hour",
-			    "%i hours",
-			    hours), hours);
-    else
-	/* TRANSLATOR: "%i %s %i %s" are "%i hours %i minutes"
-	 * Swap order with "%2$s %2$i %1$s %1$i if needed */
-	timestring = g_strdup_printf (_("%i %s %i %s"),
-			    hours, ngettext ("hour", "hours", hours),
-			    minutes, ngettext ("minute", "minutes", minutes));
-    return timestring;
-}
-
 /*
  * Refresh tooltip function for UPS and battery device only.
  */
 static void
 xfpm_battery_set_tooltip_primary (XfpmBattery *battery, GtkTooltip *tooltip)
 {
-    const gchar *battery_name;
-    gchar *tip = NULL;
+    gchar *tip;
     gchar *est_time_str = NULL;
     gchar *power_status = NULL;
     
     power_status = g_strdup_printf (battery->priv->ac_online ? _("Adaptor is online") : _("System is running on battery power"));
     
-    battery_name = battery->priv->type == XFPM_DEVICE_TYPE_BATTERY ? _("Battery") : _("UPS");
-    
     if ( battery->priv->state == XFPM_DEVICE_STATE_FULLY_CHARGED )
     {
 	if ( battery->priv->time_to_empty > 0 )
@@ -331,7 +410,7 @@ xfpm_battery_set_tooltip_primary (XfpmBattery *battery, GtkTooltip *tooltip)
 	    est_time_str = xfpm_battery_get_time_string (battery->priv->time_to_empty);
 	    tip = g_strdup_printf (_("%s\nYour %s is fully charged (%i%%).\nProvides %s runtime"), 
 				   power_status,
-				   battery_name, 
+				   battery->priv->battery_name, 
 				   battery->priv->percentage,
 				   est_time_str);
 	    g_free (est_time_str);
@@ -340,7 +419,7 @@ xfpm_battery_set_tooltip_primary (XfpmBattery *battery, GtkTooltip *tooltip)
 	{
 	    tip = g_strdup_printf (_("%s\nYour %s is fully charged (%i%%)."), 
 				   power_status,
-				   battery_name,
+				   battery->priv->battery_name,
 				   battery->priv->percentage);
 	}
     }
@@ -351,7 +430,7 @@ xfpm_battery_set_tooltip_primary (XfpmBattery *battery, GtkTooltip *tooltip)
 	    est_time_str = xfpm_battery_get_time_string (battery->priv->time_to_full);
 	    tip = g_strdup_printf (_("%s\nYour %s is charging (%i%%)\n%s until is fully charged."), 
 				   power_status,
-				   battery_name, 
+				   battery->priv->battery_name, 
 				   battery->priv->percentage, 
 				   est_time_str);
 	    g_free (est_time_str);
@@ -360,7 +439,7 @@ xfpm_battery_set_tooltip_primary (XfpmBattery *battery, GtkTooltip *tooltip)
 	{
 	    tip = g_strdup_printf (_("%s\nYour %s is charging (%i%%)."),
 				   power_status,
-				   battery_name,
+				   battery->priv->battery_name,
 				   battery->priv->percentage);
 	}
     }
@@ -369,9 +448,9 @@ xfpm_battery_set_tooltip_primary (XfpmBattery *battery, GtkTooltip *tooltip)
 	if ( battery->priv->time_to_empty != 0 )
 	{
 	    est_time_str = xfpm_battery_get_time_string (battery->priv->time_to_empty);
-	    tip = g_strdup_printf (_("%s\nYour %s is discharging (%i%%)\nestimated time left is %s."), 
+	    tip = g_strdup_printf (_("%s\nYour %s is discharging (%i%%)\nEstimated time left is %s."), 
 				   power_status,
-				   battery_name, 
+				   battery->priv->battery_name, 
 				   battery->priv->percentage, 
 				   est_time_str);
 	    g_free (est_time_str);
@@ -380,22 +459,22 @@ xfpm_battery_set_tooltip_primary (XfpmBattery *battery, GtkTooltip *tooltip)
 	{
 	    tip = g_strdup_printf (_("%s\nYour %s is discharging (%i%%)."),
 				   power_status,
-				   battery_name,
+				   battery->priv->battery_name,
 				   battery->priv->percentage);
 	}
 	
     }
     else if ( battery->priv->state == XFPM_DEVICE_STATE_PENDING_CHARGING )
     {
-	tip = g_strdup_printf (_("%s\n%s waiting to discharge (%i%%)."), power_status, battery_name, battery->priv->percentage);
+	tip = g_strdup_printf (_("%s\n%s waiting to discharge (%i%%)."), power_status, battery->priv->battery_name, battery->priv->percentage);
     }
     else if ( battery->priv->state == XFPM_DEVICE_STATE_PENDING_DISCHARGING )
     {
-	tip = g_strdup_printf (_("%s\n%s waiting to charge (%i%%)."), power_status, battery_name, battery->priv->percentage);
+	tip = g_strdup_printf (_("%s\n%s waiting to charge (%i%%)."), power_status, battery->priv->battery_name, battery->priv->percentage);
     }
     else if ( battery->priv->state == XFPM_DEVICE_STATE_EMPTY )
     {
-	tip = g_strdup_printf (_("%s\nYour %s is empty"), power_status, battery_name);
+	tip = g_strdup_printf (_("%s\nYour %s is empty"), power_status, battery->priv->battery_name);
     }
     
     gtk_tooltip_set_text (tooltip, tip);
@@ -507,13 +586,20 @@ xfpm_battery_refresh (XfpmBattery *battery, GHashTable *props)
 	g_hash_table_destroy (props);
 }
 
+static void
+xfpm_battery_button_pressed_cb (XfpmButton *button, XfpmButtonKey type, XfpmBattery *battery)
+{
+    if (type == BUTTON_BATTERY)
+	xfpm_battery_notify (battery);
+}
+
 static void
 xfpm_battery_changed_cb (DBusGProxy *proxy, XfpmBattery *battery)
 {
     GHashTable *props;
     
     props = xfpm_power_get_interface_properties (battery->priv->proxy_prop, 
-						 battery->priv->backend_iface_device);
+						 UPOWER_IFACE_DEVICE);
     
     if ( props )
 	xfpm_battery_refresh (battery, props);
@@ -655,10 +741,15 @@ xfpm_battery_init (XfpmBattery *battery)
     battery->priv->icon_prefix   = NULL;
     battery->priv->time_to_full  = 0;
     battery->priv->time_to_empty = 0;
+    battery->priv->button        = xfpm_button_new ();
     battery->priv->ac_online     = TRUE;
     
     battery->priv->sig = g_signal_connect (G_OBJECT (battery->priv->conf), "notify::" SHOW_TRAY_ICON_CFG,
 					   G_CALLBACK (xfpm_battery_tray_icon_settings_changed), battery);
+					   
+    
+    battery->priv->sig_bt = g_signal_connect (G_OBJECT (battery->priv->button), "button-pressed",
+					   G_CALLBACK (xfpm_battery_button_pressed_cb), battery);
 }
 
 static void
@@ -675,11 +766,15 @@ xfpm_battery_finalize (GObject *object)
 				    
     if ( g_signal_handler_is_connected (battery->priv->conf, battery->priv->sig ) )
 	g_signal_handler_disconnect (G_OBJECT (battery->priv->conf), battery->priv->sig);
+	
+     if ( g_signal_handler_is_connected (battery->priv->button, battery->priv->sig_bt ) )
+	g_signal_handler_disconnect (G_OBJECT (battery->priv->button), battery->priv->sig_bt);
 				    
     g_object_unref (battery->priv->proxy);
     g_object_unref (battery->priv->proxy_prop);
     g_object_unref (battery->priv->conf);
     g_object_unref (battery->priv->notify);
+    g_object_unref (battery->priv->button);
 
     G_OBJECT_CLASS (xfpm_battery_parent_class)->finalize (object);
 }
@@ -711,6 +806,42 @@ xfpm_battery_get_icon_prefix_device_enum_type (XfpmDeviceType type)
     return g_strdup (XFPM_PRIMARY_ICON_PREFIX);
 }
 
+static const gchar *
+xfpm_battery_get_name (XfpmDeviceType type)
+{
+    const gchar *name = NULL;
+    
+    switch (type)
+    {
+	case XFPM_DEVICE_TYPE_BATTERY:
+	    name = _("battery");
+	    break;
+	case XFPM_DEVICE_TYPE_UPS:
+	    name = _("UPS");
+	    break;
+	case XFPM_DEVICE_TYPE_MONITOR:
+	    name = _("monitor battery");
+	    break;
+	case XFPM_DEVICE_TYPE_MOUSE:
+	    name = _("mouse battery");
+	    break;
+	case XFPM_DEVICE_TYPE_KBD:
+	    name = _("keyboard battery");
+	    break;
+	case XFPM_DEVICE_TYPE_PDA:
+	    name = _("PDA battery");
+	    break;
+	case XFPM_DEVICE_TYPE_PHONE:
+	    name = _("Phone battery");
+	    break;
+	default:
+	    name = _("Unknown");
+	    break;
+    }
+    
+    return name;
+}
+
 GtkStatusIcon *
 xfpm_battery_new (void)
 {
@@ -724,14 +855,14 @@ xfpm_battery_new (void)
 void xfpm_battery_monitor_device (XfpmBattery *battery,
 				  DBusGProxy *proxy,
 				  DBusGProxy *proxy_prop,
-				  const gchar *backend_iface_device,
 				  XfpmDeviceType device_type)
 {
     battery->priv->type = device_type;
     battery->priv->proxy_prop = proxy_prop;
     battery->priv->proxy = proxy;
     battery->priv->icon_prefix = xfpm_battery_get_icon_prefix_device_enum_type (device_type);
-    battery->priv->backend_iface_device = backend_iface_device;
+    battery->priv->battery_name = xfpm_battery_get_name (device_type);
+    
     
     dbus_g_proxy_add_signal (proxy, "Changed", G_TYPE_INVALID);
     dbus_g_proxy_connect_signal (proxy, "Changed",
@@ -757,3 +888,17 @@ XfpmBatteryCharge xfpm_battery_get_charge (XfpmBattery *battery)
     
     return battery->priv->charge;
 }
+
+const gchar *xfpm_battery_get_battery_name (XfpmBattery *battery)
+{
+    g_return_val_if_fail (XFPM_IS_BATTERY (battery), NULL);
+    
+    return battery->priv->battery_name;
+}
+
+gchar *xfpm_battery_get_time_left (XfpmBattery *battery)
+{
+    g_return_val_if_fail (XFPM_IS_BATTERY (battery), NULL);
+    
+    return xfpm_battery_get_time_string (battery->priv->time_to_empty);
+}
