@@ -1,5 +1,5 @@
 /*
- * * Copyright (C) 2008-2009 Ali <aliov@xfce.org>
+ * * Copyright (C) 2008-2011 Ali <aliov@xfce.org>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -65,6 +65,9 @@ struct XfpmNotifyPrivate
     NotifyNotification *notification;
     NotifyNotification *critical;
     
+    gulong		critical_id;
+    gulong		notify_id;
+    
     gboolean	        supports_actions;
     gboolean		supports_sync; /*For x-canonical-private-synchronous */
 };
@@ -167,6 +170,9 @@ xfpm_notify_init (XfpmNotify *notify)
     notify->priv->notification = NULL;
     notify->priv->critical = NULL;
     
+    notify->priv->critical_id = 0;
+    notify->priv->notify_id   = 0;
+    
     notify->priv->monitor = xfpm_dbus_monitor_new ();
     xfpm_dbus_monitor_add_service (notify->priv->monitor, DBUS_BUS_SESSION, "org.freedesktop.Notifications");
     g_signal_connect (notify->priv->monitor, "service-connection-changed",
@@ -209,13 +215,29 @@ xfpm_notify_new_notification_internal (const gchar *title, const gchar *message,
 {
     NotifyNotification *n;
     
+#ifdef NOTIFY_CHECK_VERSION
+#if NOTIFY_CHECK_VERSION (0, 7, 0) 
+    n = notify_notification_new (title, message, NULL);
+#else
     n = notify_notification_new (title, message, NULL, NULL);
+#endif
+#else
+    n = notify_notification_new (title, message, NULL, NULL);
+#endif
+
     
     if ( icon_name )
     	xfpm_notify_set_notification_icon (n, icon_name);
-	
+
+#ifdef NOTIFY_CHECK_VERSION
+#if !NOTIFY_CHECK_VERSION (0, 7, 0) 
     if ( icon )
     	notify_notification_attach_to_status_icon (n, icon);
+#else
+    if ( icon )
+    	notify_notification_attach_to_status_icon (n, icon);
+#endif
+#endif
 	
     notify_notification_set_urgency (n, (NotifyUrgency)urgency);
     
@@ -249,6 +271,12 @@ xfpm_notify_show (NotifyNotification *n)
 static void
 xfpm_notify_close_notification (XfpmNotify *notify )
 {
+    if (notify->priv->notify_id != 0)
+    {
+	g_source_remove (notify->priv->notify_id);
+	notify->priv->notify_id = 0;
+    }
+    
     if ( notify->priv->notification )
     {
     	if (!notify_notification_close (notify->priv->notification, NULL))
@@ -335,7 +363,7 @@ void xfpm_notify_present_notification (XfpmNotify *notify, NotifyNotification *n
 	notify->priv->notification = n;
     }
     
-    g_idle_add ((GSourceFunc) xfpm_notify_show, n);
+    notify->priv->notify_id = g_idle_add ((GSourceFunc) xfpm_notify_show, n);
 }
 
 void xfpm_notify_critical (XfpmNotify *notify, NotifyNotification *n)
@@ -349,13 +377,20 @@ void xfpm_notify_critical (XfpmNotify *notify, NotifyNotification *n)
     g_signal_connect (G_OBJECT (n), "closed", 
 		      G_CALLBACK (xfpm_notify_close_critical_cb), notify);
 		      
-    g_idle_add ((GSourceFunc) xfpm_notify_show, n);
+    notify->priv->critical_id = g_idle_add ((GSourceFunc) xfpm_notify_show, n);
 }
 
 void xfpm_notify_close_critical (XfpmNotify *notify)
 {
     g_return_if_fail (XFPM_IS_NOTIFY (notify));
     
+    
+    if (notify->priv->critical_id != 0)
+    {
+	g_source_remove (notify->priv->critical_id);
+	notify->priv->critical_id = 0;
+    }
+    
     if ( notify->priv->critical )
     {
     	if (!notify_notification_close (notify->priv->critical, NULL))
