@@ -1,5 +1,5 @@
 /*
- * * Copyright (C) 2009 Ali <aliov@xfce.org>
+ * * Copyright (C) 2009-2011 Ali <aliov@xfce.org>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -34,6 +34,7 @@
 
 #include "xfpm-power.h"
 #include "xfpm-dbus.h"
+#include "xfpm-dpms.h"
 #include "xfpm-battery.h"
 #include "xfpm-xfconf.h"
 #include "xfpm-notify.h"
@@ -46,9 +47,9 @@
 #include "xfpm-common.h"
 #include "xfpm-power-common.h"
 #include "xfpm-config.h"
-#include "xfpm-enum-glib.h"
 #include "xfpm-debug.h"
 #include "xfpm-enum-types.h"
+#include "egg-idletime.h"
 
 static void xfpm_power_finalize     (GObject *object);
 
@@ -79,6 +80,12 @@ struct XfpmPowerPrivate
     XfpmXfconf      *conf;
     GtkStatusIcon   *adapter_icon;
     
+    XfpmBatteryCharge overall_state;
+    gboolean         critical_action_done;
+    
+    XfpmPowerMode    power_mode;
+    EggIdletime     *idletime;
+    
     gboolean	     inhibited;
     
     XfpmNotify	    *notify;
@@ -297,7 +304,7 @@ xfpm_power_report_error (XfpmPower *power, const gchar *error, const gchar *icon
     }
     
     xfpm_notify_show_notification (power->priv->notify, 
-				   _("Xfce power manager"), 
+				   _("Power Manager"), 
 				   error, 
 				   icon_name,
 				   10000,
@@ -421,13 +428,47 @@ xfpm_power_tray_exit_activated_cb (gpointer data)
 			       _("Quit"),
 			       _("All running instances of the power manager will exit"),
 			       "%s",
-			        _("Quit Xfce power manager?"));
+			        _("Quit the power manager?"));
     if ( ret )
     {
 	xfpm_quit ();
     }
 }
 
+
+static void
+xfpm_power_change_mode (XfpmPower *power, XfpmPowerMode mode)
+{
+    XfpmDpms *dpms;
+    
+    power->priv->power_mode = mode;
+    
+    dpms = xfpm_dpms_new ();
+    xfpm_dpms_refresh (dpms);
+    g_object_unref (dpms);
+    
+    if (mode == XFPM_POWER_MODE_NORMAL)
+    {
+	EggIdletime *idletime;
+	idletime = egg_idletime_new ();
+	egg_idletime_alarm_reset_all (idletime);
+    
+	g_object_unref (idletime);
+    }
+}
+
+static void
+xfpm_power_normal_mode_cb (XfpmPower *power)
+{
+    xfpm_power_change_mode (power, XFPM_POWER_MODE_NORMAL);
+}
+
+static void
+xfpm_power_presentation_mode_cb (XfpmPower *power)
+{
+    xfpm_power_change_mode (power, XFPM_POWER_MODE_PRESENTATION);
+}
+
 static void
 xfpm_power_show_tray_menu (XfpmPower *power, 
 			 GtkStatusIcon *icon, 
@@ -435,7 +476,7 @@ xfpm_power_show_tray_menu (XfpmPower *power,
 			 guint activate_time,
 			 gboolean show_info_item)
 {
-    GtkWidget *menu, *mi, *img;
+    GtkWidget *menu, *mi, *img, *subm;
 
     menu = gtk_menu_new();
 
@@ -490,7 +531,6 @@ xfpm_power_show_tray_menu (XfpmPower *power,
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
     // Power information
-    
     mi = gtk_image_menu_item_new_with_label (_("Power Information"));
     img = gtk_image_new_from_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU);
     gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), img);
@@ -503,6 +543,41 @@ xfpm_power_show_tray_menu (XfpmPower *power,
     gtk_widget_show (mi);
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
     
+    /** 
+     * Power Mode
+     **/
+    /* TRANSLATOR: Mode here is the power profile (presentation, power save, normal) */
+    mi = gtk_image_menu_item_new_with_label (_("Mode"));
+    img = gtk_image_new_from_icon_name (XFPM_AC_ADAPTER_ICON, GTK_ICON_SIZE_MENU);
+    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), img);
+    gtk_widget_set_sensitive (mi,TRUE);
+    gtk_widget_show (mi);
+    
+    subm = gtk_menu_new ();
+    gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), subm);
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+    
+    /* Normal*/
+    mi = gtk_check_menu_item_new_with_label (_("Normal"));
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), power->priv->power_mode == XFPM_POWER_MODE_NORMAL);
+    gtk_widget_set_sensitive (mi,TRUE);
+    
+    g_signal_connect_swapped (mi, "activate",
+			      G_CALLBACK (xfpm_power_normal_mode_cb), power);
+    gtk_widget_show (mi);
+    gtk_menu_shell_append (GTK_MENU_SHELL (subm), mi);
+    
+    /* Normal*/
+    mi = gtk_check_menu_item_new_with_label (_("Presentation"));
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), power->priv->power_mode == XFPM_POWER_MODE_PRESENTATION);
+    gtk_widget_set_sensitive (mi, TRUE);
+    
+    g_signal_connect_swapped (mi, "activate",
+			      G_CALLBACK (xfpm_power_presentation_mode_cb), power);
+    gtk_widget_show (mi);
+    gtk_menu_shell_append (GTK_MENU_SHELL (subm), mi);
+    
+    
     mi = gtk_separator_menu_item_new ();
     gtk_widget_show (mi);
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
@@ -517,7 +592,7 @@ xfpm_power_show_tray_menu (XfpmPower *power,
     mi = gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
     gtk_widget_set_sensitive (mi, TRUE);
     gtk_widget_show (mi);
-    g_signal_connect (mi, "activate", G_CALLBACK (xfpm_about), _("Xfce Power Manager"));
+    g_signal_connect (mi, "activate", G_CALLBACK (xfpm_about), _("Power Manager"));
     
     gtk_menu_shell_append (GTK_MENU_SHELL(menu), mi);
     
@@ -652,7 +727,7 @@ xfpm_power_show_critical_action_notification (XfpmPower *power, XfpmBattery *bat
 	      
     n = 
 	xfpm_notify_new_notification (power->priv->notify, 
-				      _("Xfce power manager"), 
+				      _("Power Manager"), 
 				      message, 
 				      gtk_status_icon_get_icon_name (GTK_STATUS_ICON (battery)),
 				      20000,
@@ -688,7 +763,7 @@ xfpm_power_show_critical_action_gtk (XfpmPower *power)
     message = _("System is running on low power. "\
                "Save your work to avoid losing data");
     
-    dialog = gtk_dialog_new_with_buttons (_("Xfce Power Manager"), NULL, GTK_DIALOG_MODAL,
+    dialog = gtk_dialog_new_with_buttons (_("Power Manager"), NULL, GTK_DIALOG_MODAL,
                                           NULL);
     
     gtk_dialog_set_default_response (GTK_DIALOG (dialog),
@@ -783,7 +858,7 @@ xfpm_power_process_critical_action (XfpmPower *power, XfpmShutdownRequest req)
 }
 
 static void
-xfpm_power_system_on_low_power (XfpmPower *power, XfpmBattery *battery)
+xfpm_power_system_on_critical_power (XfpmPower *power, XfpmBattery *battery)
 {
     XfpmShutdownRequest critical_action;
     
@@ -800,7 +875,15 @@ xfpm_power_system_on_low_power (XfpmPower *power, XfpmBattery *battery)
     }
     else
     {
-	xfpm_power_process_critical_action (power, critical_action);
+	if (power->priv->critical_action_done == FALSE)
+	{
+	    power->priv->critical_action_done = TRUE;
+	    xfpm_power_process_critical_action (power, critical_action);
+	}
+	else
+	{
+	    xfpm_power_show_critical_action (power, battery);
+	}
     }
 }
 
@@ -816,9 +899,18 @@ xfpm_power_battery_charge_changed_cb (XfpmBattery *battery, XfpmPower *power)
     
     XFPM_DEBUG_ENUM (current_charge, XFPM_TYPE_BATTERY_CHARGE, "Current system charge status");
     
+    if (current_charge == power->priv->overall_state)
+	return;
+    
+    if (current_charge >= XFPM_BATTERY_CHARGE_LOW)
+	power->priv->critical_action_done = FALSE;
+    
+    power->priv->overall_state = current_charge;
+    
     if ( current_charge == XFPM_BATTERY_CHARGE_CRITICAL && power->priv->on_battery)
     {
-	xfpm_power_system_on_low_power (power, battery);
+	xfpm_power_system_on_critical_power (power, battery);
+	
 	power->priv->on_low_battery = TRUE;
 	g_signal_emit (G_OBJECT (power), signals [LOW_BATTERY_CHANGED], 0, power->priv->on_low_battery);
 	return;
@@ -840,7 +932,7 @@ xfpm_power_battery_charge_changed_cb (XfpmBattery *battery, XfpmPower *power)
 	{
 	    if ( notify )
 		xfpm_notify_show_notification (power->priv->notify, 
-					       _("Xfce power manager"), 
+					       _("Power Manager"), 
 					       _("System is running on low power"), 
 					       gtk_status_icon_get_icon_name (GTK_STATUS_ICON (battery)),
 					       10000,
@@ -851,16 +943,29 @@ xfpm_power_battery_charge_changed_cb (XfpmBattery *battery, XfpmPower *power)
 	}
 	else if ( battery_charge == XFPM_BATTERY_CHARGE_LOW )
 	{
-	    
 	    if ( notify )
+	    {
+		gchar *msg;
+		gchar *time_str;
+		
+		const gchar *battery_name = xfpm_battery_get_battery_name (battery);
+		
+		time_str = xfpm_battery_get_time_left (battery);
+		
+		msg = g_strdup_printf (_("Your %s charge level is low\nEstimated time left %s"), battery_name, time_str);
+		
+		
 		xfpm_notify_show_notification (power->priv->notify, 
-					       _("Xfce power manager"), 
-					       _("Battery charge level is low"), 
+					       _("Power Manager"), 
+					       msg, 
 					       gtk_status_icon_get_icon_name (GTK_STATUS_ICON (battery)),
 					       10000,
 					       FALSE,
 					       XFPM_NOTIFY_NORMAL,
 					       GTK_STATUS_ICON (battery));
+		g_free (msg);
+		g_free (time_str);
+	    }
 	}
     }
     
@@ -913,7 +1018,6 @@ xfpm_power_add_device (XfpmPower *power, const gchar *object_path)
 	xfpm_battery_monitor_device (XFPM_BATTERY (battery), 
 				     proxy, 
 				     proxy_prop, 
-				     UPOWER_IFACE_DEVICE, 
 				     device_type);
 
 	g_hash_table_insert (power->priv->hash, g_strdup (object_path), battery);
@@ -1220,6 +1324,9 @@ xfpm_power_init (XfpmPower *power)
     power->priv->auth_suspend    = TRUE;
     power->priv->dialog          = NULL;
     power->priv->adapter_icon    = NULL;
+    power->priv->overall_state   = XFPM_BATTERY_CHARGE_OK;
+    power->priv->critical_action_done = FALSE;
+    power->priv->power_mode      = XFPM_POWER_MODE_NORMAL;
     
     power->priv->inhibit = xfpm_inhibit_new ();
     power->priv->notify  = xfpm_notify_new ();
@@ -1428,6 +1535,13 @@ gboolean xfpm_power_has_battery (XfpmPower *power)
     return ret;
 }
 
+XfpmPowerMode  xfpm_power_get_mode (XfpmPower *power)
+{
+    g_return_val_if_fail (XFPM_IS_POWER (power), XFPM_POWER_MODE_NORMAL);
+    
+    return power->priv->power_mode;
+}
+
 /*
  * 
  * DBus server implementation for org.freedesktop.PowerManagement
